# Lang.pm (HCT::Lang).
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

=head1 NAME

HCT::Lang - Class of lanugages. Loads and provides connection with HCT.

=head1 DESCRIPTION

This package is responsible for loading HDL into the HCT environment.

HCT::Lang will automatically call the method C<new> of each language package to get handler.

=cut

package HCT::Lang;

our $VERSION = '0.2.1';

BEGIN
{
	-d $ENV{'HCTHOME'} or die "\$HCTHOME not correct.\n";
}

use strict;
use warnings;

# Global subroutine AUTOLOAD to provide global reports.
sub AUTOLOAD
{
	return HCT::AUTOLOAD(
		(caller),
		our $AUTOLOAD,
		@_
	);
}

use HCT::Constants;
use HCT::File;

=head1 VARIABLES

=over 4
=cut

use vars qw(
	%HANDLERS
	$INITSCRIPT
); 

=item B<%HANDLERS>

Stores HANDLERS of the languages, where key is a language name, and the value 
is a language object.

=cut

%HANDLERS = EMPTY_LIST;

=item B<$INITSCRIPT>

Contents path to the init-script file.

=cut

$INITSCRIPT = join '/' => ($ENV{HCTHOME}, 'HCT', 'lang', 'init.pl');

=head1 METHODS

Note that all C<HCT::Lang> functionality is method based; use C<HCT::Lang-E<gt>get_handler> rather
than C<HCT::Lang::get_handler>.

=over 4
=cut

=item B<init ()>

Initializes all languages defined in C<@languages>. Each language will be loaded and 
corresponding handler will be created in C<%HANDLERS>. 

=cut

sub init
{
	HCT::ERROR ("init-script file '$INITSCRIPT' does not exist.\n")
		unless is_file ($INITSCRIPT);

	eval "require '$INITSCRIPT'";
	HCT::ERROR ("can not use init-script '$INITSCRIPT', the reason: $@\n")
		if ($@);
}

=item B<langlist ()>

Returns list of avaliable languages from C<%HANDLERS>.

=cut

sub langlist ($)
{
	my $class = shift;
	return keys %HANDLERS;
}

=item B<is_defined (LANGUAGE)>

Takes language and returns true if such language defined and false, if no. 

=cut

sub is_defined ($)
{
	my $class = shift;
	my $lang = HCT::Lang->unify_name (shift);

	if (defined $HANDLERS{$lang}) {
		return TRUE;
	}

	return FALSE;
}

=item B<unify_name (LANGUAGE)>

Takes language name (LANGUAGE) and returns unified name.

=cut

sub unify_name ($)
{
	my ($class, $lang) = (shift, shift);
	return lc ($lang);
}

=item B<get_handler (LANGUAGE)>

Takes language name (LANGUAGE) and returns language handler or undef.

=cut

sub get_handler ($)
{
	my $class = shift;
	my $lang = HCT::Lang->unify_name (shift);

	if (HCT::Lang->is_defined ($lang)) {
		return $HANDLERS{$lang};
	}

	return undef;
}

=item B<load (LANGUAGE, FILE, PACKAGE)>

Takes language name (LANGUAGE), file name (FILE) and package name (PACKAGE) as arguments and try to init
language handler: will be called method C<new>.

=cut

sub load
{
	no strict 'refs';

	my $class = shift;
	my $name = shift;
	my $unified_name = HCT::Lang->unify_name ($name);
	my $file = join '/' => ($ENV{HCTHOME}, 'HCT', 'lang', shift);
	my $package = shift;

	# Checking the file...
	unless (is_file ($file)) {
		HCT::DEBUG ("Language $name, file '$file' does not exist.\n");
		return;
	}

	eval "require '$file'";
	if ($@) {
		HCT::DEBUG ("Can not require '$file'. The reason:\n\n$@\n");
		return;
	}

	# Checking, does the language already evailable...
	if (HCT::Lang->is_defined ($name)) {
		HCT::DEBUG ("Language $name was already defined.\n");
		return ;
	}

	# Creating language handler...
	if ($package->can ('new')) {
		$HANDLERS{$unified_name} = $package->new ( );
	} else {
		HCT::DEBUG ("Can not call method 'new' of package $package.\n");
	}

	HCT::DEBUG ("Language $name has been initialized.\n");

	# Provide file associations
	HCT::Lang->provide_file_associations ($name, $package);

	use strict 'refs';
}

=item B<provide_file_associations (LANGUAGE, PACKAGE)>

Takes language name (LANGUAGE) and language package name (PACKAGE) and saves file associations of this language to the
global list C<%HCT::FILE_ASSOCIATIONS>.

=cut

sub provide_file_associations
{
	no strict 'refs';
	
	my $class = shift;
	my $foo = 'FILE_ASSOCIATIONS';
	my $lang = shift;
	my $package = shift;
	my $associations = ${"${package}::"}{$foo};

	# if variable with file associations is not defined
	unless (defined $associations) {
		HCT::DEBUG ("Can not provide file associations for $lang, maybe you forgot to fill variable $foo.\n");
		return;
	}

	# if array fith file associations is empty
	if (scalar (@$associations) == 0) {
		HCT::DEBUG ("No file associations for language $lang.\n");
		return;
	}

	if (defined ${"HCT::"}{$foo}) {
		FILE_ASSOCIATION: for (@$associations) {
			if (defined (my $prev_lang = ${"HCT::"}{$foo}{$_})) {
				HCT::DEBUG ("'$_' was already defined for language $prev_lang. Replaced.\n");
				next FILE_ASSOCIATION;
			} else {
				${"HCT::"}{$foo}{$_} = $lang;
			}
		}
	}

	use strict 'refs';
}

=back
=cut

1;

